local f = CreateFrame("Frame")

local defaults = {
    hideRealm = true,
	hideRealmParty = true,
	hideRealmNP = false,
    hideName = false,
	hideNameParty = false,
    hideRole = false,
	hideRoleParty = false,
    hideRaidMan = false,
    hideNameArena = false,
	hideNameArenaFriendly = false,
}

function f:OnEvent(event, addOnName)
    if addOnName == "HideRealmNames" then
        HideRealmNamesDB = HideRealmNamesDB or CopyTable(defaults)
        self.db = HideRealmNamesDB
        self:InitializeOptions()

        -- Hide realm on raid frame
        if self.db.hideRealm then
            hooksecurefunc("CompactUnitFrame_UpdateName", function(frame)
                if frame and not frame:IsForbidden() then
                    local frame_name = frame:GetName()
                    if frame_name and (frame_name:match("^PartyMemberFrame%d") or frame_name:match("^CompactRaidFrame%d") or frame_name:match("^CompactRaidGroup%dMember%d") or frame_name:match("^CompactPartyFrameMember%d")) and frame.unit and frame.name then
                        if UnitIsPlayer(frame.unit) then
                            local unit_name = GetUnitName(frame.unit, true)
                            if unit_name then
                                frame.name:SetText(unit_name:match("[^-]+"))
                            end
                        end
                    end
                end
            end)
        end

		-- Hide realm on party frame 
		if self.db.hideRealmParty then
			if WOW_PROJECT_ID == WOW_PROJECT_MAINLINE then
				hooksecurefunc("UnitFrame_Update", function(frame)
					for frame in PartyFrame.PartyMemberFramePool:EnumerateActive() do
						local unit_name = GetUnitName(frame.unit, true)
						if unit_name then
							frame.name:SetText(unit_name:match("[^-]+"))
						end
					end
				end)
			elseif WOW_PROJECT_ID == WOW_PROJECT_CLASSIC or WOW_PROJECT_WRATH_CLASSIC then
				hooksecurefunc("UnitFrame_Update", function(frame)
					if frame and not frame:IsForbidden() then
						local frame_name = frame:GetName()
						if frame_name and frame_name:match("^PartyMemberFrame%d") and frame.unit and frame.name then
							local unit_name = GetUnitName(frame.unit, true)
							if unit_name then
								frame.name:SetText(unit_name:match("[^-]+"))
							end
						end
					end
				end)
			end
		end		

		-- Hide realm from nameplates
		if self.db.hideRealmNP then
			hooksecurefunc("CompactUnitFrame_UpdateName", function(frame)
				local isInInstance, instanceType = IsInInstance()
				if not isInInstance or instanceType == "none" and ShouldShowName(frame) then
					if frame.optionTable.colorNameBySelection then
						frame.name:SetText(GetUnitName(frame.unit))
					end
				end
			end)
		end		

        -- Hide player names on raid frame
        if self.db.hideName then
            hooksecurefunc("CompactUnitFrame_UpdateName", function(frame)
				if frame.optionTable == DefaultCompactUnitFrameOptions then
					frame.name:Hide()
				end
			end)			
        end

		-- Hide player names on party frame
		if self.db.hideNameParty then
			if WOW_PROJECT_ID == WOW_PROJECT_MAINLINE then
				for frame in PartyFrame.PartyMemberFramePool:EnumerateActive() do
					frame.Name:Hide()		
				end
			elseif WOW_PROJECT_ID == WOW_PROJECT_CLASSIC or WOW_PROJECT_WRATH_CLASSIC then
				for i = 1, 4 do 
					_G["PartyMemberFrame"..i.."Name"]:Hide()
				end
			end
		end
		
        -- Hide role icons on raid frame
        if self.db.hideRole then
			hooksecurefunc("CompactUnitFrame_UpdateName", function(frame)
				if frame.optionTable == DefaultCompactUnitFrameOptions then
					frame.roleIcon:Hide()
				end
			end)			
		end

		-- Hide role icons on party frame
		if self.db.hideRoleParty then
			if WOW_PROJECT_ID == WOW_PROJECT_MAINLINE then
				for frame in PartyFrame.PartyMemberFramePool:EnumerateActive() do
					frame.PartyMemberOverlay.RoleIcon:Hide()			
				end
			elseif WOW_PROJECT_ID == WOW_PROJECT_CLASSIC or WOW_PROJECT_ID == WOW_PROJECT_WRATH_CLASSIC then
				hooksecurefunc("UnitFrame_Update", function(frame)
					if frame.optionTable == DefaultCompactUnitFrameOptions then
						frame.roleIcon:Hide()
					end
				end)
			end
		end

        -- Hide Raid Manager Panel
		if self.db.hideRaidMan then
			--[=[
			if WOW_PROJECT_ID == WOW_PROJECT_MAINLINE then
				CompactRaidFrameManager:UnregisterAllEvents()
				CompactRaidFrameManager:HookScript("OnShow", function(s) s:Hide() end)
				CompactRaidFrameManager:Hide()
			elseif WOW_PROJECT_ID == WOW_PROJECT_CLASSIC or WOW_PROJECT_WRATH_CLASSIC then
			]=]
				local texture1ist1 = { CompactRaidFrameManager:GetRegions() }
				local texture1ist2 = select(1, CompactRaidFrameManagerToggleButton:GetRegions())
				local function SetShown(self)
					if self:IsMouseOver() and self.collapsed then 
						texture1ist2:SetAlpha(1)
						for _,v in pairs(texture1ist1) do
							v:SetAlpha(1)
						end
					elseif not self:IsMouseOver() and self.collapsed then
						texture1ist2:SetAlpha(0)
						for _,v in pairs(texture1ist1) do
							v:SetAlpha(0)
						end
					end
				end
				CompactRaidFrameManager:HookScript("OnEnter", SetShown)
				CompactRaidFrameManager:HookScript("OnLeave", SetShown)
				CompactRaidFrameManagerToggleButton:HookScript("OnEnter", function(self) SetShown(self:GetParent()) end)
				CompactRaidFrameManagerToggleButton:HookScript("OnLeave", function(self) SetShown(self:GetParent()) end)
				SetShown(CompactRaidFrameManager)
			end
		--end
		
        -- Replace enemy player names with numbers on arena namplates
        if self.db.hideNameArena then
			local U = UnitIsUnit
			hooksecurefunc("CompactUnitFrame_UpdateName", function(F)
				if IsActiveBattlefieldArena() and F.unit:find("nameplate") then
					for i = 1, 5 do
						if U(F.unit, "arena" .. i) then
							F.name:SetText(i)
							F.name:SetTextColor(1, 0, 0)
							break
						end
					end
				end
			end)
		end
		
		-- Replace friendly player names with numbers on arena namplates
		if self.db.hideNameArenaFriendly then
			local U = UnitIsUnit
			hooksecurefunc("CompactUnitFrame_UpdateName", function(F)
				if IsActiveBattlefieldArena() and F.unit:find("nameplate") then
					for i = 1, 5 do
						if U(F.unit, "party" .. i) then
							F.name:SetText(i)
							F.name:SetTextColor(0, 1, 0)
							break
						end
					end
				end
			end)
		end
    end
end

f:RegisterEvent("ADDON_LOADED")
f:SetScript("OnEvent", f.OnEvent)

function f:InitializeOptions()
    self.panel = CreateFrame("Frame")
    self.panel.name = "Hide Realm Names"

    local function createFontString(parent, font, justify, point1, relativeTo, point2, x, y, text)
        local fs = parent:CreateFontString(nil, 'ARTWORK', font)
        fs:SetJustifyH(justify)
        fs:SetPoint(point1, relativeTo, point2, x, y)
        fs:SetText(text)
        return fs
    end

    local function createCheckButton(parent, point1, relativeTo, x, y, text, dbKey)
        local cb = CreateFrame("CheckButton", nil, parent, "InterfaceOptionsCheckButtonTemplate")
        cb:SetPoint(point1, relativeTo, x, y)
        cb.Text:SetText(text)
        cb:HookScript("OnClick", function(_, btn, down)
            self.db[dbKey] = cb:GetChecked()
        end)
        cb:SetChecked(self.db[dbKey])
        return cb
    end

    local title = createFontString(self.panel, 'GameFontNormalHuge', 'LEFT', 'TOPLEFT', self.panel, 'TOPLEFT', 16, -16, self.panel.name)
    local subtitle = createFontString(self.panel, 'GameFontNormal', 'LEFT', 'TOPLEFT', title, 'BOTTOMLEFT', 0, -8, 'All changes will take effect after a UI reload.')
    local subheader_1 = createFontString(self.panel, 'GameFontNormalLarge', 'LEFT', 'TOPLEFT', subtitle, 'BOTTOMLEFT', 0, -40, 'Group Frame Options')
    
	local subheader_realm = createFontString(self.panel, 'GameFontHighlightLarge', 'LEFT', 'TOPLEFT', subheader_1, 'BOTTOMLEFT', 0, -10, 'Hide Realm Names')
    local cb_hide_realms = createCheckButton(self.panel, "TOPLEFT", subheader_realm, 0, -20, "Raid", "hideRealm")
    local cb_hide_realms_party = createCheckButton(self.panel, "TOPLEFT", cb_hide_realms, 0, -25, "Party", "hideRealmParty")

    local subheader_names = createFontString(self.panel, 'GameFontHighlightLarge', 'LEFT', 'TOPLEFT', cb_hide_realms, 'BOTTOMLEFT', 0, -40, 'Hide Player Names')
    local cb_hide_names = createCheckButton(self.panel, "TOPLEFT", subheader_names, 0, -20, "Raid", "hideName")
    local cb_hide_names_party = createCheckButton(self.panel, "TOPLEFT", cb_hide_names, 0, -25, "Party", "hideNameParty")

    local subheader_roles = createFontString(self.panel, 'GameFontHighlightLarge', 'LEFT', 'TOPLEFT', subheader_realm, 'TOPLEFT', 200, 0, 'Hide Role Icons')
    local cb_hide_roles = createCheckButton(self.panel, "TOPLEFT", subheader_roles, 0, -20, "Raid", "hideRole")
	local cb_hide_roles_party = createCheckButton(self.panel, "TOPLEFT", cb_hide_roles, 0, -25, "Party", "hideRoleParty")

	local subheader_raidman  = createFontString(self.panel, 'GameFontHighlightLarge', 'LEFT', 'TOPLEFT', subheader_names, 'TOPLEFT', 200, 0, 'Raid Frame Manager (left side panel)')
	local cb_hide_raidman  = createCheckButton(self.panel, "TOPLEFT", subheader_raidman, 0, -20, "Hidden", "hideRaidMan")

	local subheader_2 = createFontString(self.panel, 'GameFontNormalLarge', 'LEFT', 'TOPLEFT', subheader_1, 'BOTTOMLEFT', 0, -190, 'Nameplate Options')

	local subheader_nameplate  = createFontString(self.panel, 'GameFontHighlightLarge', 'LEFT', 'TOPLEFT', subheader_2, 'BOTTOMLEFT', 0, -10, 'Hide Realm (outside instances only)')
	local cb_hide_realms_np  = createCheckButton(self.panel, "TOPLEFT", subheader_nameplate, 0, -20, "Enabled", "hideRealmNP")

	local subheader_arena_name  = createFontString(self.panel, 'GameFontHighlightLarge', 'LEFT', 'TOPLEFT', cb_hide_realms_np, 'BOTTOMLEFT', 0, -10, 'Replace Names with Numbers on Arena Nameplates (Arena 123)')
	local cb_hide_arena  = createCheckButton(self.panel, "TOPLEFT", subheader_arena_name, 0, -20, "Enemy", "hideNameArena")
	local cb_hide_arena_friendly  = createCheckButton(self.panel, "TOPLEFT", cb_hide_arena, 0, -25, "Friendly", "hideNameArenaFriendly")
	
	local btn = CreateFrame("Button", nil, self.panel, "UIPanelButtonTemplate")
	btn:SetPoint("TOPLEFT", cb_hide_arena, 0, -65)
	btn:SetText("Save and Reload")
	btn:SetWidth(120)
	btn:SetScript("OnClick", function()
        ReloadUI();
	end)

    InterfaceOptions_AddCategory(self.panel)
end

SLASH_HRN1 = "/hrn"
SLASH_HRN2 = "/hiderealmnames"
SLASH_HRNRL1 = "/rl"

SlashCmdList.HRN = function(msg, editBox)
	InterfaceOptionsFrame_OpenToCategory(f.panel)
SlashCmdList.HRNRL = ReloadUI
end